__all__ = ("ANSI_RESET", "ANSI_COLOR")

TEMPLATE = "{}"

ANSI_ESCAPE = "\x1b"
ANSI_START = "["
ANSI_STOP = "m"

ANSI_SEPARATOR = ";"

ANSI_RESET_COMMAND = "0"
ANSI_COLOR_COMMAND = "38"

ANSI_RGB = "2"

ANSI_RESET = ANSI_ESCAPE + ANSI_START + ANSI_RESET_COMMAND + ANSI_STOP
ANSI_COLOR = (
    ANSI_ESCAPE
    + ANSI_START
    + ANSI_COLOR_COMMAND
    + ANSI_SEPARATOR
    + ANSI_RGB
    + ANSI_SEPARATOR
    + TEMPLATE
    + ANSI_SEPARATOR
    + TEMPLATE
    + ANSI_SEPARATOR
    + TEMPLATE
    + ANSI_STOP
)
