"""Core color functionality."""

__description__ = "Core color functionality."
__url__ = "https://github.com/nekitdev/colors-core"

__title__ = "colors"
__author__ = "nekitdev"
__license__ = "MIT"
__version__ = "1.0.0"

from colors.core import Color

__all__ = ("Color",)
